﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CarExceptionsTest
{
    class Car
    {
        private string _brand;
        private string _model;
        private string _regNo;
        private int _horsePower;
        private int _doors;

        public string Brand
        {
            get => _brand;
            set
            {
                if (string.IsNullOrWhiteSpace(value))
                    throw new ArgumentException("Brand må ikke være null, emty eller kun bestå af blanktegn");
                _brand = value;
            }
        }

        public string Model
        {
            get => _model;
            set
            {
                if (value == null) throw new ArgumentNullException("Brand må ikke være null");
                if (value.Equals("")) throw new ArgumentException("Brand skal have en værdi");
                if (value.Length > 20) throw new ArgumentException("Brand må max indeholde 20 karakterer");
                _model = value;
            } 
        }

        public string RegNo
        {
            get => _regNo;
            set
            {
                if (value == null) throw new ArgumentNullException("RegNo må ikke være null");
                if (value.Equals("")) throw new ArgumentException("RegNo skal have en værdi");
                char[] temp = value.ToCharArray();
                string tal = value.Substring(2, 5);
                
                if (!char.IsLetter(temp[0]) || !char.IsLetter(temp[1]) || int.TryParse(tal, out int i)  )
                _regNo = value;
            }
        } 

        public int HorsePower
        {
            get => _horsePower;
            set => _horsePower = value;
        }

        public int Doors
        {
            get => _doors;
            set => _doors = value;
        }

        public Car()
        {
            
        }

        public Car(string brand, string model, string regNo, int horsePower, int doors)
        {
            Brand = brand;
            Model = model;
            RegNo = regNo;
            HorsePower = horsePower;
            Doors = doors;
        }

        protected bool Equals(Car other)
        {
            return Brand == other.Brand && Model == other.Model && RegNo == other.RegNo && HorsePower == other.HorsePower && Doors == other.Doors;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((Car) obj);
        }

        public override int GetHashCode()
        {
            return HashCode.Combine(Brand, Model, RegNo, HorsePower, Doors);
        }

        public override string ToString()
        {
            return $"Brand: {Brand}, Model: {Model}, RegNo: {RegNo}, HorsePower: {HorsePower}, Doors: {Doors}";
        }
    }
}
